    % Observer Parameters for DF200
    % Parameters for constants a7 and a9
    % Physical constants
    global t_sim Sigma_SB R_Ex_Gas cp_Ex_Gas T_amb  C_s cp_SCR p_amb  Ea_fastSCR k_fastSCR  SCR_D Epsilon_Rad m_SCR k_ads  NH3_Slip_Optim x0
    global Epsilon  k_des  mdot_exgas ndot_NH3 T_exgas Init_SCR_Temp M_NH3 n_Cell R A_rad A_SCR L_SCR V_SCR NO_Slip_Optim NO2_Slip_Optim
    global alpha dh_NH3  Ea_oxN2 k_oxN2 NOX_Slip_Optim ndot_no ndot_no2 Ea_stdSCR k_stdSCR k_slowSCR Ea_slowSCR Temp_Optim K2 K1  A1 A2 A3 n_Cell_T
    global  A_Pipe m_total c_pipe h_pipe Init_pipe_temp mflow_urea zeta_us cp_h2Og cp_urea cp_h20l rv_h20   h_external  h_internal h_convection Epsilon_pipe D_inner  L_pipe
    global h_radiation

    R=8.314;               % universal gas constant [J/mol/K]
    M_NH3=17*1e-3;         % molar mass of NH3 [kg/mol]
    Sigma_SB = 5.67*1e-8;  % Stefan-Boltzmann constant [W/m^2/K^4]
    
    %% 
    A1=[4.11262e6];       % Constants for temp. dep of exhaust gas
    A2=[1.92461e-5];
    A3=[0.168476];


    % Gas
    
    R_Ex_Gas=287;          % exhaust gas constant [J/kg/K]
    cp_Ex_Gas=1035.16;% specific heat of exhaust gas [J/kg/K]

    % Number of Cells considered for the SCR Model
    
    n_Cell=3; 
    n_Cell_T=9;
    
    K1=[0.7333];   % Constants for temp. dep of heat capacity for SCR
    K2=[4.214e-4]; 
   
    % Convertor Dimensions in inches
     L_cat=11;
     D_cat=12;
    
    L_SCR= 2.54*L_cat*1e-2 ; % Length of SCR [m]
    
    SCR_D=  2.54*D_cat*1e-2; % converter diameter [m]
    
    N_Cat=1; % Number of catalyst
    
    V_SCR= N_Cat*SCR_D^2*pi*L_SCR/4; % volume of Catalytic converter [m^3]
    
    Rho_Catalyst=1714;% 1714(600) 1838.6(300) for DW3811
    
    Epsilon=0.6468;% 0.6468(600) 0.6218(300) 0.6325(400)  for DW3811
    
    m_SCR=Rho_Catalyst*V_SCR*(1-Epsilon);%  mass of SCR converter [kg]
  

    % Ambient Temperature

    T_amb=298; %[K]

    Epsilon_Rad=0; % radiative emissivity of converter taken as zero assuming it is insulated
    
    cp_SCR=600;% specific heat of converter [J/kg/K] 
    
    % SCR system geometry calculations

    A_rad= pi*L_SCR*SCR_D;                    % area for radiative heat transfer of converter [m^2]
    
    A_SCR=(SCR_D^2/4)*pi;                     % frontal area [m^2]
 
    p_amb=1.013*1e5; % Ambient Pressure [Pa]
    
    %% Optimized values of parameters (estimates for DW3811)
%     
    C_s=146.211;
    Ea_fastSCR=6.8591e4*0 ; % 6.8591e4Activation Energy Fast SCR Reaction 1.5*1e2;
    k_fastSCR=  4.185e11*0 ;% Rate Constant SCR Reaction NO+NO2+2NH3-->2N2+3H20 653.64

k_stdSCR=1.5637e7*0 ;
Ea_stdSCR=6.3724e4*0 ;

k_slowSCR=2.2893e10 *0 ;
Ea_slowSCR=1.0675e5*0 ;
  
    k_ads=   5.79666880636476 ;%

      
%Desorption of Ammonia

    k_des =1098322.93678757 ; % Rate Constant desorption of ammonia NH3*--> NH3; 
    
% To calculate the activation energy for desorption 

    alpha=   0.1871 ;
    dh_NH3=  81716.7974257233 ;
       
    
% Oxidation of Ammonia
    Ea_oxN2=1.8116e5*0 ; 
    k_oxN2= 1.1843e12*0 ;
    
    
optim_type = 'StandardSCR'
switch optim_type
    
    case 'TPD'
%####################################################################################

        %% Use for the TPD Test and Uncomment
        %Simulation time
        t_sim=[1:4303]';

        % Initial SCR Temperature

        Init_SCR_Temp=150+273;

        load TPD_Input.mat

        mdot_exgas = TPD_Input(:,2);

        ndot_NH3 = TPD_Input(:,3);
        ndot_no=ndot_NH3*0;
        ndot_no2=ndot_NH3*0;

        T_exgas =TPD_Input(:,1)+273;

        load TPD_out.mat

        NH3_Slip_Optim = TPD_out(:,1);
        NO_Slip_Optim = zeros(size(NH3_Slip_Optim));
        NO2_Slip_Optim= zeros(size(NH3_Slip_Optim));
    
    case 'StandardSCR'

        % Uncomment for estimating SCR reaction and oxidation reaction parameters
        % Simulation time
        t_sim=[1:17000]';

        % Initial SCR Temperature

        Init_SCR_Temp=325+273;

        % Mass flow of exhaust gas
        mdot_eg1 = ones(2500,1)*0.3 ;
        mdot_eg2 = ones(4500,1)*0.5;
        mdot_eg3 = ones(2500,1)*0.6;
        mdot_eg4 = ones(2500,1)*0.2;
        mdot_eg5 = ones(3000,1)*0.6;
        mdot_eg6 = ones(2000,1)*0.4;
        mdot_exgas = [mdot_eg1' mdot_eg2' mdot_eg3' mdot_eg4' mdot_eg5' mdot_eg6']'; 

        exhaustgas_moles = mdot_exgas*1000/29; % moles of exhuast gas

        Temp1 = 325*ones(4500,1);
        Temp2 = 350*ones(2500,1);
        Temp3 = 375*ones(2500,1);
        Temp4 = 400*ones(2500,1);
        Temp5 = 425*ones(3000,1);
        Temp6 = 450*ones(2000,1);

        T_exgas = [Temp1' Temp2' Temp3' Temp4' Temp5' Temp6']'+273;


        %  Standard SCR

        NH3_ppm=[1000 0 1400 0 1600 0 1600 0 800 0 1400 0 1000];

        NH3_ppm1 = ones(2000,1)*NH3_ppm(1); 
        NH3_ppm2 = ones(500,1)*NH3_ppm(2); 
        NH3_ppm3 = ones(2000,1)*NH3_ppm(3); 
        NH3_ppm4 = ones(500,1)*NH3_ppm(4); 
        NH3_ppm5 = ones(2000,1)*NH3_ppm(5); 
        NH3_ppm6 = ones(500,1)*NH3_ppm(6); 
        NH3_ppm7 = ones(2000,1)*NH3_ppm(7); 
        NH3_ppm8 = ones(500,1)*NH3_ppm(8); 
        NH3_ppm9 = ones(2000,1)*NH3_ppm(9); 
        NH3_ppm10 = ones(500,1)*NH3_ppm(10);
        NH3_ppm11 = ones(2000,1)*NH3_ppm(11); 
        NH3_ppm12 = ones(500,1)*NH3_ppm(12); 
        NH3_ppm13 = ones(2000,1)*NH3_ppm(13); 

        NH3_ppmtot = [NH3_ppm1' NH3_ppm2' NH3_ppm3' NH3_ppm4' NH3_ppm5' NH3_ppm6' NH3_ppm7' NH3_ppm8' NH3_ppm9' NH3_ppm10' NH3_ppm11' NH3_ppm12' NH3_ppm13']';

        ndot_NH3 = NH3_ppmtot.*exhaustgas_moles/10^6;

        NO_ppm=[1000 0 1200 0 1500 0 1000 0 800 0 1600 0 400];

        NO_ppm1 = ones(2000,1)*NO_ppm(1);
        NO_ppm2 = ones(500,1)*NO_ppm(2); 
        NO_ppm3 = ones(2000,1)*NO_ppm(3); 
        NO_ppm4 = ones(500,1)*NO_ppm(4); 
        NO_ppm5 = ones(2000,1)*NO_ppm(5); 
        NO_ppm6 = ones(500,1)*NO_ppm(6); 
        NO_ppm7 = ones(2000,1)*NO_ppm(7); 
        NO_ppm8 = ones(500,1)*NO_ppm(8); 
        NO_ppm9 = ones(2000,1)*NO_ppm(9); 
        NO_ppm10 = ones(500,1)*NO_ppm(10); 
        NO_ppm11 = ones(2000,1)*NO_ppm(11); 
        NO_ppm12 = ones(500,1)*NO_ppm(12); 
        NO_ppm13 = ones(2000,1)*NO_ppm(13); 

        NO_ppmtot = [NO_ppm1' NO_ppm2' NO_ppm3' NO_ppm4' NO_ppm5' NO_ppm6' NO_ppm7' NO_ppm8' NO_ppm9' NO_ppm10' NO_ppm11' NO_ppm12' NO_ppm13']';
        ndot_no=NO_ppmtot.*exhaustgas_moles/10^6;
        ndot_no2=ndot_no*0;

        load std_scr.mat

        NH3_Slip_Optim = std_scr(:,3);
        NO_Slip_Optim = std_scr(:,1);
       NO2_Slip_Optim= std_scr(:,2);
    
    otherwise


%% slow SCR

% NH3_ppm=[1000 0 1400 0 1600 0 1600 0 800 0 1400 0 1000];
% 
% NH3_ppm1 = ones(2000,1)*NH3_ppm(1); 
% NH3_ppm2 = ones(500,1)*NH3_ppm(2); 
% NH3_ppm3 = ones(2000,1)*NH3_ppm(3); 
% NH3_ppm4 = ones(500,1)*NH3_ppm(4); 
% NH3_ppm5 = ones(2000,1)*NH3_ppm(5); 
% NH3_ppm6 = ones(500,1)*NH3_ppm(6); 
% NH3_ppm7 = ones(2000,1)*NH3_ppm(7); 
% NH3_ppm8 = ones(500,1)*NH3_ppm(8); 
% NH3_ppm9 = ones(2000,1)*NH3_ppm(9); 
% NH3_ppm10 = ones(500,1)*NH3_ppm(10);
% NH3_ppm11 = ones(2000,1)*NH3_ppm(11); 
% NH3_ppm12 = ones(500,1)*NH3_ppm(12); 
% NH3_ppm13 = ones(2000,1)*NH3_ppm(13); 
% 
% NH3_ppmtot = [NH3_ppm1' NH3_ppm2' NH3_ppm3' NH3_ppm4' NH3_ppm5' NH3_ppm6' NH3_ppm7' NH3_ppm8' NH3_ppm9' NH3_ppm10' NH3_ppm11' NH3_ppm12' NH3_ppm13']';
% 
% ndot_NH3 = NH3_ppmtot.*exhaustgas_moles/10^6;

% NO2_ppm=[1000 0 1200 0 1500 0 1000 0 800 0 1600 0 400];

% NO2_ppm1 = ones(2000,1)*NO2_ppm(1);
% NO2_ppm2 = ones(500,1)*NO2_ppm(2); 
% NO2_ppm3 = ones(2000,1)*NO2_ppm(3); 
% NO2_ppm4 = ones(500,1)*NO2_ppm(4); 
% NO2_ppm5 = ones(2000,1)*NO2_ppm(5); 
% NO2_ppm6 = ones(500,1)*NO2_ppm(6); 
% NO2_ppm7 = ones(2000,1)*NO2_ppm(7); 
% NO2_ppm8 = ones(500,1)*NO2_ppm(8); 
% NO2_ppm9 = ones(2000,1)*NO2_ppm(9); 
% NO2_ppm10 = ones(500,1)*NO2_ppm(10); 
% NO2_ppm11 = ones(2000,1)*NO2_ppm(11); 
% NO2_ppm12 = ones(500,1)*NO2_ppm(12); 
% NO2_ppm13 = ones(2000,1)*NO2_ppm(13); 

% NO2_ppmtot = [NO2_ppm1' NO2_ppm2' NO2_ppm3' NO2_ppm4' NO2_ppm5' NO2_ppm6' NO2_ppm7' NO2_ppm8' NO2_ppm9' NO2_ppm10' NO2_ppm11' NO2_ppm12' NO2_ppm13']';
% ndot_no2=NO2_ppmtot.*exhaustgas_moles/10^6;
% ndot_no=ndot_no2*0;

% load slow_scr.mat
% 
% NH3_Slip_Optim = slow_scr(:,3);
% NO_Slip_Optim = slow_scr(:,1);
% NO2_Slip_Optim= slow_scr(:,2);

%%  fast SCR
%  NH3_ppm=[1000 0 1200 0 1500 0 1000 0 800 0 1600 0 1600];
% 
% NH3_ppm1 = ones(2000,1)*NH3_ppm(1); 
% NH3_ppm2 = ones(500,1)*NH3_ppm(2); 
% NH3_ppm3 = ones(2000,1)*NH3_ppm(3); 
% NH3_ppm4 = ones(500,1)*NH3_ppm(4); 
% NH3_ppm5 = ones(2000,1)*NH3_ppm(5); 
% NH3_ppm6 = ones(500,1)*NH3_ppm(6); 
% NH3_ppm7 = ones(2000,1)*NH3_ppm(7); 
% NH3_ppm8 = ones(500,1)*NH3_ppm(8); 
% NH3_ppm9 = ones(2000,1)*NH3_ppm(9); 
% NH3_ppm10 = ones(500,1)*NH3_ppm(10);
% NH3_ppm11 = ones(2000,1)*NH3_ppm(11); 
% NH3_ppm12 = ones(500,1)*NH3_ppm(12); 
% NH3_ppm13 = ones(2000,1)*NH3_ppm(13); 
% 
% NH3_ppmtot = [NH3_ppm1' NH3_ppm2' NH3_ppm3' NH3_ppm4' NH3_ppm5' NH3_ppm6' NH3_ppm7' NH3_ppm8' NH3_ppm9' NH3_ppm10' NH3_ppm11' NH3_ppm12' NH3_ppm13']';
% 
% ndot_NH3 = NH3_ppmtot.*exhaustgas_moles/10^6;

% ndot_no=ndot_NH3*0.5;
% ndot_no2=ndot_NH3*0.5;

% load fast_scr.mat
% 
% NH3_Slip_Optim = fast_scr(:,3);
% NO_Slip_Optim = fast_scr(:,1);
% NO2_Slip_Optim= fast_scr(:,2);

%%  Oxidation

%  NH3_ppm=[1000 0 1200 0 1500 0 1000 0 800 0 1600 0 1600];
% 
% NH3_ppm1 = ones(2000,1)*NH3_ppm(1); 
% NH3_ppm2 = ones(500,1)*NH3_ppm(2); 
% NH3_ppm3 = ones(2000,1)*NH3_ppm(3); 
% NH3_ppm4 = ones(500,1)*NH3_ppm(4); 
% NH3_ppm5 = ones(2000,1)*NH3_ppm(5); 
% NH3_ppm6 = ones(500,1)*NH3_ppm(6); 
% NH3_ppm7 = ones(2000,1)*NH3_ppm(7); 
% NH3_ppm8 = ones(500,1)*NH3_ppm(8); 
% NH3_ppm9 = ones(2000,1)*NH3_ppm(9); 
% NH3_ppm10 = ones(500,1)*NH3_ppm(10);
% NH3_ppm11 = ones(2000,1)*NH3_ppm(11); 
% NH3_ppm12 = ones(500,1)*NH3_ppm(12); 
% NH3_ppm13 = ones(2000,1)*NH3_ppm(13); 
% 
% NH3_ppmtot = [NH3_ppm1' NH3_ppm2' NH3_ppm3' NH3_ppm4' NH3_ppm5' NH3_ppm6' NH3_ppm7' NH3_ppm8' NH3_ppm9' NH3_ppm10' NH3_ppm11' NH3_ppm12' NH3_ppm13']';
% 
% ndot_NH3 = NH3_ppmtot.*exhaustgas_moles/10^6;
% 
% ndot_no=ndot_NH3*0.5;
% ndot_no2=ndot_NH3*0.5;
% 
% load ox_scr.mat
% 
% NH3_Slip_Optim = ox_scr(:,3);
% NO_Slip_Optim = ox_scr(:,1);
% NO2_Slip_Optim=ox_scr(:,2);
end

switch optim_type
    case 'TPD'
 %% Parameter Estimation Routine
 
        options=optimset('Display','iter');

         x0 = [1e7 78e3 6 150 0.1871];  % Starting guess for TPD

        [x,fval,exitflag,output]=fminsearchbnd(@Costfcn,x0,[0 25e3 0 90 0],[+inf +250e3 10 280 1],options,optim_type);

    case 'StandardSCR'
        options=optimset('Display','iter');

         x0 = [1.5637e7 6.3724e4];  % Starting guess

        [x,fval,exitflag,output]=fminsearchbnd(@Costfcn,x0,[0 25e3],[+inf +250e3],options,optim_type);
    
%% 
end


%   lsqnonlin
    x0 = [1.2e17 162e3];
    options = optimoptions(@lsqnonlin,'Algorithm','trust-region-reflective','Display','iter');
    [x,resnorm,residual,exitflag,output,lambda,jacobian] = lsqnonlin(@LSQ_J,x0,[],[],options);  % May have to be modified to work (look at cost function)

